/******************************************************************************
 * The copyright in this software is being made available under the BSD        *
 * License, included below. This software may be subject to other third party  *
 * and contributor rights, including patent rights, and no such rights are     *
 * granted under this license.                                                 *
 *    <OWNER> = ISO                                                            *
 *    <ORGANIZATION> = ISO                                                     *
 *    <YEAR> = 2015                                                            *
 *    Copyright (c) 2012-2015, ISO                                             *
 *                                                                             *
 * All rights reserved.                                                        *
 * Redistribution and use in source and binary forms, with or without          *
 * modification, are permitted provided that the following conditions are met: *
 *       * Redistributions of source code must retain the above copyright      *
 *         notice,this list of conditions and the following disclaimer.        *
 *       * Redistributions in binary form must reproduce the above             *
 *         copyright notice, this list of conditions and the following         *
 *         disclaimer in the documentation and/or other materials provided     *
 *         with the distribution.                                              *
 *       * Neither the name of the <ORGANIZATION> nor the names of its         *
 *         contributors may be used to endorse or promote products derived     *
 *         from this software without specific prior written permission.       *
 *                                                                             *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" *
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE  *
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE   *
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR         *
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF        *
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS    *
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN     *
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)     *
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE  *
 * POSSIBILITY OF SUCH DAMAGE.                                                 *
 ******************************************************************************/

#ifndef JH_MEMMAP_H
#define JH_MEMMAP_H
#include <stdlib.h>
#include <fcntl.h>
#include <sys/stat.h>
#ifndef WIN32
#include <sys/mman.h>
#include <unistd.h>
#else
#include <stdio.h>
#endif
#include "string.h"
#include "jh_host.h"

/* returns 0 (null) if open, fstat or mmap failed otherwise address of memory mapped file */
static void* memmap(const char* filename, size_t* size) {
#ifndef WIN32 /* Android and unix support mmap */
    void* buf = MAP_FAILED;
    int fd = -1;
    struct stat st = {0};
    fd = open(filename, O_RDONLY);
    if (fd != -1 && fstat(fd, &st) == 0) {
        *size = st.st_size;
        buf = mmap(0, *size, PROT_READ, MAP_SHARED, fd, 0);
    }
    if (fd != -1) {
        close(fd);
    }
    return buf == MAP_FAILED ? 0 : buf;
#else
    unsigned char* buf = NULL;
    FILE *fd = NULL;
    size_t bytes = 0;
    fd = fopen(filename, "rb");
    if (fd != NULL) {
        if (fseek(fd, 0, SEEK_END)) {
            goto cleanup;
        }
        *size=ftell(fd);
        buf = (unsigned char *) jh_malloc(*size);
        if (NULL == buf) {
            goto cleanup;
        }
        if (fseek(fd, 0, SEEK_SET)) {
            goto cleanup;
        }
        bytes = fread(buf,1,*size,fd);
        if (bytes != *size) {
            fputs("jh_memmap: error while reading file", stderr);
            jh_free(buf);
            buf = NULL;
            goto cleanup;
        }
    }
cleanup:
    if (NULL != fd) {
        fclose(fd);
    }
    return buf;
#endif
}

static int memunmap(void* buf, size_t size) {
#ifndef WIN32 /* Android and unix supports mmap */
    return buf == null || munmap(buf, size) != -1;
#else
    free(buf);
    return buf == null;
#endif
}
#endif /*JH_MEMMAP_H*/
